@@ -6,14 +6,12 @@
 #include "CppGridUtils.h"
 #include "SimplexProjection.h"
 #include "SMap.h"
-#include "IntersectionCardinality.h"
+#include "IntersectionalCardinality.h"
 #include "PatternCausality.h"
 #include <RcppThread.h>
 
-// [[Rcpp::depends(RcppThread)]]
-
 /*
- * Evaluates prediction performance of different combinations of embedding dimensions, number of nearest neighbors and 
+ * Evaluates prediction performance of different combinations of embedding dimensions, number of nearest neighbors and
  * tau values for grid data using simplex projection forecasting.
  *
  * Parameters:
@@ -105,7 +103,7 @@ std::vector<std::vector<double>> Simplex4Grid(const std::vector<std::vector<doub
 }
 
 /*
- * Evaluates prediction performance of different combinations of embedding dimensions, number of nearest neighbors and 
+ * Evaluates prediction performance of different combinations of embedding dimensions, number of nearest neighbors and
  * tau values for grid data using simplex projection forecasting (composite embeddings version).
  */
 std::vector<std::vector<double>> Simplex4GridCom(const std::vector<std::vector<double>>& source,
@@ -294,11 +292,11 @@ std::vector<std::vector<double>> SMap4GridCom(const std::vector<std::vector<doub
 }
 
 /**
- * @brief Evaluate intersection cardinality (IC) for spatial grid data.
+ * @brief Evaluate intersectional cardinality (IC) for spatial grid data.
  *
- * This function computes the intersection cardinality between the k-nearest neighbors
+ * This function computes the intersectional cardinality between the k-nearest neighbors
  * of grid-embedded source and target spatial variables, across a range of embedding dimensions (E),
- * neighborhood sizes (b) and spatial lag step (tau). The result is an AUC (Area Under the Curve) 
+ * neighborhood sizes (b) and spatial lag step (tau). The result is an AUC (Area Under the Curve)
  * score for each (E, tau) pair that quantifies the directional similarity or interaction between
  * the spatial fields.
  *
@@ -360,7 +358,7 @@ std::vector<std::vector<double>> IC4Grid(const std::vector<std::vector<double>>&
     for (int t : taus)
       unique_ETau.emplace_back(e, t);
 
-  std::vector<std::vector<double>> result(unique_ETau.size() * bs.size(), 
+  std::vector<std::vector<double>> result(unique_ETau.size() * bs.size(),
                                           std::vector<double>(5));
 
   size_t max_num_neighbors = 0;
@@ -407,7 +405,7 @@ std::vector<std::vector<double>> IC4Grid(const std::vector<std::vector<double>>&
         const size_t k = static_cast<size_t>(bs[j]);
 
         // run cross mapping
-        std::vector<IntersectionRes> res = IntersectionCardinalitySingle(
+        std::vector<IntersectionRes> res = IntersectionalCardinalitySingle(
           nx,ny,lib_indices.size(),lib_indices,valid_pred,k,n_excluded_sizet,threads_sizet,0
         );
 
@@ -460,7 +458,7 @@ std::vector<std::vector<double>> IC4Grid(const std::vector<std::vector<double>>&
         const size_t k = static_cast<size_t>(bs[j]);
 
         // run cross mapping
-        std::vector<IntersectionRes> res = IntersectionCardinalitySingle(
+        std::vector<IntersectionRes> res = IntersectionalCardinalitySingle(
           nx,ny,lib_indices.size(),lib_indices,valid_pred,k,n_excluded_sizet,threads_sizet,1
         );
 
