@@ -12,7 +12,10 @@
 #include "CppGridUtils.h"
 #include "SimplexProjection.h"
 #include "SMap.h"
-#include <RcppThread.h>
+// Note: <RcppThread.h> is intentionally excluded from this header to avoid
+//       unnecessary Rcpp dependencies and potential header inclusion order
+//       issues (e.g., R.h being included before Rcpp headers). It should only
+//       be included in the corresponding .cpp implementation file.
 
 /**
  * Perform Grid-based Geographical Convergent Cross Mapping (GCCM) for a single library size and pred indice.
