@@ -229,31 +229,33 @@ Rcpp::NumericVector RcppGenGridSymbolization(const Rcpp::NumericMatrix& mat,
 
   // Convert lib to a fundamental C++ data type
   int lib_dim = lib.ncol();
-  std::vector<std::pair<int, int>> lib_std(lib.nrow());
+  std::vector<std::pair<int, int>> lib_std;
+  lib_std.reserve(lib.nrow());
 
   if (lib_dim == 1){
     for (int i = 0; i < lib.nrow(); ++i) {
       std::vector<int> rowcolnum = RowColFromGrid(lib(i, 0) - 1, numCols);
-      lib_std[i] = std::make_pair(rowcolnum[0], rowcolnum[1]);
+      lib_std.emplace_back(rowcolnum[0], rowcolnum[1]);
     }
   } else {
     for (int i = 0; i < lib.nrow(); ++i) {
-      lib_std[i] = std::make_pair(lib(i, 0) - 1, lib(i, 1) - 1);
+      lib_std.emplace_back(lib(i, 0) - 1, lib(i, 1) - 1);
     }
   }
 
   // Convert pred to a fundamental C++ data type
   int pred_dim = pred.ncol();
-  std::vector<std::pair<int, int>> pred_std(pred.nrow());
+  std::vector<std::pair<int, int>> pred_std;
+  pred_std.reserve(pred.nrow());
 
   if (pred_dim == 1){
     for (int i = 0; i < pred.nrow(); ++i) {
       std::vector<int> rowcolnum = RowColFromGrid(pred(i, 0) - 1, numCols);
-      pred_std[i] = std::make_pair(rowcolnum[0], rowcolnum[1]);
+      pred_std.emplace_back(rowcolnum[0], rowcolnum[1]);
     }
   } else {
     for (int i = 0; i < pred.nrow(); ++i) {
-      pred_std[i] = std::make_pair(pred(i, 0) - 1, pred(i, 1) - 1);
+      pred_std.emplace_back(pred(i, 0) - 1, pred(i, 1) - 1);
     }
   }
 
@@ -478,14 +480,9 @@ Rcpp::NumericVector RcppFNN4Grid(
   int numRows = mat.nrow();
   int numCols = mat.ncol();
   std::vector<std::vector<double>> cppMat(numRows, std::vector<double>(numCols));
-
-  double validCellNum = 0;
   for (int r = 0; r < numRows; ++r) {
     for (int c = 0; c < numCols; ++c) {
       cppMat[r][c] = mat(r, c);
-      if (!std::isnan(mat(r, c))){
-        validCellNum += 1;
-      }
     }
   }
 
@@ -498,6 +495,7 @@ Rcpp::NumericVector RcppFNN4Grid(
   int n_predcol = pred.ncol();
 
   std::vector<size_t> lib_std;
+  lib_std.reserve(lib.nrow());
   if (n_libcol == 1){
     for (int i = 0; i < lib.nrow(); ++i) {
       // disallow lib indices to point to vectors with NaN
@@ -516,6 +514,7 @@ Rcpp::NumericVector RcppFNN4Grid(
   }
 
   std::vector<size_t> pred_std;
+  pred_std.reserve(pred.nrow());
   if (n_predcol == 1){
     for (int i = 0; i < pred.nrow(); ++i) {
       // disallow pred indices to point to vectors with NaN
@@ -608,8 +607,10 @@ Rcpp::NumericMatrix RcppSimplex4Grid(const Rcpp::NumericMatrix& source,
   }
 
   // Initialize lib_indices and pred_indices
-  std::vector<int> pred_indices;
   std::vector<int> lib_indices;
+  lib_indices.reserve(lib.nrow());
+  std::vector<int> pred_indices;
+  pred_indices.reserve(pred.nrow());
 
   // Convert lib and pred (1-based in R) to 0-based indices and set corresponding positions to true
   int currow;
@@ -761,8 +762,10 @@ Rcpp::NumericMatrix RcppSMap4Grid(const Rcpp::NumericMatrix& source,
   }
 
   // Initialize lib_indices and pred_indices
-  std::vector<int> pred_indices;
   std::vector<int> lib_indices;
+  lib_indices.reserve(lib.nrow());
+  std::vector<int> pred_indices;
+  pred_indices.reserve(pred.nrow());
 
   // Convert lib and pred (1-based in R) to 0-based indices and set corresponding positions to true
   int currow;
@@ -923,8 +926,10 @@ Rcpp::NumericMatrix RcppMultiView4Grid(const Rcpp::NumericMatrix& xMatrix,
   }
 
   // Initialize lib_indices and pred_indices
-  std::vector<int> pred_indices;
   std::vector<int> lib_indices;
+  lib_indices.reserve(lib.nrow());
+  std::vector<int> pred_indices;
+  pred_indices.reserve(pred.nrow());
 
   // Convert lib and pred (1-based in R) to 0-based indices and set corresponding positions to true
   int lib_col = lib.ncol();
@@ -1125,8 +1130,10 @@ Rcpp::NumericMatrix RcppIC4Grid(const Rcpp::NumericMatrix& source,
   }
 
   // Initialize lib_indices and pred_indices
-  std::vector<size_t> pred_indices;
   std::vector<size_t> lib_indices;
+  lib_indices.reserve(lib.nrow());
+  std::vector<size_t> pred_indices;
+  pred_indices.reserve(pred.nrow());
 
   // Convert lib and pred (1-based in R) to 0-based indices and set corresponding positions to true
   int currow;
@@ -1251,8 +1258,10 @@ Rcpp::NumericMatrix RcppPC4Grid(const Rcpp::NumericMatrix& source,
   }
 
   // Initialize lib_indices and pred_indices
-  std::vector<size_t> pred_indices;
   std::vector<size_t> lib_indices;
+  lib_indices.reserve(lib.nrow());
+  std::vector<size_t> pred_indices;
+  pred_indices.reserve(pred.nrow());
 
   // Convert lib and pred (1-based in R) to 0-based indices and set corresponding positions to true
   int currow;
@@ -1306,8 +1315,9 @@ Rcpp::NumericMatrix RcppPC4Grid(const Rcpp::NumericMatrix& source,
   std::vector<int> E_std = Rcpp::as<std::vector<int>>(E);
   std::vector<int> tau_std = Rcpp::as<std::vector<int>>(tau);
 
-  // Check the validity of the neignbor numbers
+  // Check the validity of the neighbor numbers
   std::vector<int> b_std;
+  b_std.reserve(b.size());
   for (int i = 0; i < b.size(); ++i){
     if (b[i] > static_cast<int>(lib_indices.size())) {
       Rcpp::stop("Neighbor numbers count out of acceptable range at position %d (value: %d)", i + 1, b[i]);
@@ -1394,7 +1404,10 @@ Rcpp::NumericMatrix RcppGCCM4Grid(
   // Convert libsizes to a fundamental C++ data type
   int libsizes_dim = libsizes.ncol();
   std::vector<int> libsizes_cpp1;
+  libsizes_cpp1.reserve(libsizes.nrow());
   std::vector<std::vector<int>> libsizes_cpp2(2);
+  libsizes_cpp2[0].reserve(libsizes.nrow());
+  libsizes_cpp2[1].reserve(libsizes.nrow());
   if (libsizes_dim == 1){
     for (int i = 0; i < libsizes.nrow(); ++i) {
       libsizes_cpp1.push_back(libsizes(i, 0));
@@ -1409,7 +1422,9 @@ Rcpp::NumericMatrix RcppGCCM4Grid(
   // Convert lib to a fundamental C++ data type
   int lib_dim = lib.ncol();
   std::vector<int> lib_cpp1;
-  std::vector<std::pair<int, int>> lib_cpp2(lib.nrow());
+  lib_cpp1.reserve(lib.nrow());
+  std::vector<std::pair<int, int>> lib_cpp2;
+  lib_cpp2.reserve(lib.nrow());
   if (libsizes_dim == 1){
     if (lib_dim == 1){
       for (int i = 0; i < lib.nrow(); ++i) {
@@ -1438,7 +1453,7 @@ Rcpp::NumericMatrix RcppGCCM4Grid(
         // disallow lib indices to point to vectors with NaN
         if (!std::isnan(xMatrix_cpp[rowcolnum[0]][rowcolnum[1]]) &&
             !std::isnan(yMatrix_cpp[rowcolnum[0]][rowcolnum[1]])){
-          lib_cpp2[i] = std::make_pair(rowcolnum[0], rowcolnum[1]);
+          lib_cpp2.emplace_back(rowcolnum[0], rowcolnum[1]);
         }
       }
     } else {
@@ -1446,7 +1461,7 @@ Rcpp::NumericMatrix RcppGCCM4Grid(
         // disallow lib indices to point to vectors with NaN
         if (!std::isnan(xMatrix_cpp[lib(i, 0) - 1][lib(i, 1) - 1]) &&
             !std::isnan(yMatrix_cpp[lib(i, 0) - 1][lib(i, 1) - 1])){
-          lib_cpp2[i] = std::make_pair(lib(i, 0) - 1, lib(i, 1) - 1);
+          lib_cpp2.emplace_back(lib(i, 0) - 1, lib(i, 1) - 1);
         }
       }
     }
@@ -1455,7 +1470,9 @@ Rcpp::NumericMatrix RcppGCCM4Grid(
   // Convert pred to a fundamental C++ data type
   int pred_dim = pred.ncol();
   std::vector<int> pred_cpp1;
-  std::vector<std::pair<int, int>> pred_cpp2(pred.nrow());
+  pred_cpp1.reserve(pred.nrow());
+  std::vector<std::pair<int, int>> pred_cpp2;
+  pred_cpp2.reserve(pred.nrow());
   if (libsizes_dim == 1){
     if (pred_dim == 1){
       for (int i = 0; i < pred.nrow(); ++i) {
@@ -1484,7 +1501,7 @@ Rcpp::NumericMatrix RcppGCCM4Grid(
         // disallow pred indices to point to vectors with NaN
         if (!std::isnan(xMatrix_cpp[rowcolnum[0]][rowcolnum[1]]) &&
             !std::isnan(yMatrix_cpp[rowcolnum[0]][rowcolnum[1]])){
-          pred_cpp2[i] = std::make_pair(rowcolnum[0], rowcolnum[1]);
+          pred_cpp2.emplace_back(rowcolnum[0], rowcolnum[1]);
         }
       }
     } else {
@@ -1492,7 +1509,7 @@ Rcpp::NumericMatrix RcppGCCM4Grid(
         // disallow pred indices to point to vectors with NaN
         if (!std::isnan(xMatrix_cpp[pred(i, 0) - 1][pred(i, 1) - 1]) &&
             !std::isnan(yMatrix_cpp[pred(i, 0) - 1][pred(i, 1) - 1])){
-          pred_cpp2[i] = std::make_pair(pred(i, 0) - 1, pred(i, 1) - 1);
+          pred_cpp2.emplace_back(pred(i, 0) - 1, pred(i, 1) - 1);
         }
       }
     }
@@ -1638,7 +1655,10 @@ Rcpp::NumericMatrix RcppSCPCM4Grid(
   // Convert libsizes to a fundamental C++ data type
   int libsizes_dim = libsizes.ncol();
   std::vector<int> libsizes_cpp1;
+  libsizes_cpp1.reserve(libsizes.nrow());
   std::vector<std::vector<int>> libsizes_cpp2(2);
+  libsizes_cpp2[0].reserve(libsizes.nrow());
+  libsizes_cpp2[1].reserve(libsizes.nrow());
   if (libsizes_dim == 1){
     for (int i = 0; i < libsizes.nrow(); ++i) {
       libsizes_cpp1.push_back(libsizes(i, 0));
@@ -1653,7 +1673,9 @@ Rcpp::NumericMatrix RcppSCPCM4Grid(
   // Convert lib to a fundamental C++ data type
   int lib_dim = lib.ncol();
   std::vector<int> lib_cpp1;
-  std::vector<std::pair<int, int>> lib_cpp2(lib.nrow());
+  lib_cpp1.reserve(lib.nrow());
+  std::vector<std::pair<int, int>> lib_cpp2;
+  lib_cpp2.reserve(lib.nrow());
   if (libsizes_dim == 1){
     if (lib_dim == 1){
       for (int i = 0; i < lib.nrow(); ++i) {
@@ -1682,7 +1704,7 @@ Rcpp::NumericMatrix RcppSCPCM4Grid(
         // disallow lib indices to point to vectors with NaN
         if (!std::isnan(xMatrix_cpp[rowcolnum[0]][rowcolnum[1]]) &&
             !std::isnan(yMatrix_cpp[rowcolnum[0]][rowcolnum[1]])){
-          lib_cpp2[i] = std::make_pair(rowcolnum[0], rowcolnum[1]);
+          lib_cpp2.emplace_back(rowcolnum[0], rowcolnum[1]);
         }
       }
     } else {
@@ -1690,7 +1712,7 @@ Rcpp::NumericMatrix RcppSCPCM4Grid(
         // disallow lib indices to point to vectors with NaN
         if (!std::isnan(xMatrix_cpp[lib(i, 0) - 1][lib(i, 1) - 1]) &&
             !std::isnan(yMatrix_cpp[lib(i, 0) - 1][lib(i, 1) - 1])){
-          lib_cpp2[i] = std::make_pair(lib(i, 0) - 1, lib(i, 1) - 1);
+          lib_cpp2.emplace_back(lib(i, 0) - 1, lib(i, 1) - 1);
         }
       }
     }
@@ -1699,7 +1721,9 @@ Rcpp::NumericMatrix RcppSCPCM4Grid(
   // Convert pred to a fundamental C++ data type
   int pred_dim = pred.ncol();
   std::vector<int> pred_cpp1;
-  std::vector<std::pair<int, int>> pred_cpp2(pred.nrow());
+  pred_cpp1.reserve(pred.nrow());
+  std::vector<std::pair<int, int>> pred_cpp2;
+  pred_cpp2.reserve(pred.nrow());
   if (libsizes_dim == 1){
     if (pred_dim == 1){
       for (int i = 0; i < pred.nrow(); ++i) {
@@ -1728,7 +1752,7 @@ Rcpp::NumericMatrix RcppSCPCM4Grid(
         // disallow pred indices to point to vectors with NaN
         if (!std::isnan(xMatrix_cpp[rowcolnum[0]][rowcolnum[1]]) &&
             !std::isnan(yMatrix_cpp[rowcolnum[0]][rowcolnum[1]])){
-          pred_cpp2[i] = std::make_pair(rowcolnum[0], rowcolnum[1]);
+          pred_cpp2.emplace_back(rowcolnum[0], rowcolnum[1]);
         }
       }
     } else {
@@ -1736,7 +1760,7 @@ Rcpp::NumericMatrix RcppSCPCM4Grid(
         // disallow pred indices to point to vectors with NaN
         if (!std::isnan(xMatrix_cpp[pred(i, 0) - 1][pred(i, 1) - 1]) &&
             !std::isnan(yMatrix_cpp[pred(i, 0) - 1][pred(i, 1) - 1])){
-          pred_cpp2[i] = std::make_pair(pred(i, 0) - 1, pred(i, 1) - 1);
+          pred_cpp2.emplace_back(pred(i, 0) - 1, pred(i, 1) - 1);
         }
       }
     }
@@ -1869,7 +1893,7 @@ Rcpp::List RcppGCMC4Grid(
   }
 
   // Convert Rcpp NumericMatrix to std::vector<std::vector<double>>
-  double validCellNum = 0;
+  size_t validCellNum = 0;
   std::vector<std::vector<double>> yMatrix_cpp(yMatrix.nrow(), std::vector<double>(yMatrix.ncol()));
   for (int i = 0; i < yMatrix.nrow(); ++i) {
     for (int j = 0; j < yMatrix.ncol(); ++j) {
@@ -1934,7 +1958,7 @@ Rcpp::List RcppGCMC4Grid(
     }
   }
 
-  if (b <= 3 || b > validCellNum) {
+  if (b <= 3 || static_cast<size_t>(b) > validCellNum) {
     Rcpp::stop("k must be greater than 3 and no larger than the number of non-NA values.\n"
                "An empirical rule of thumb is to set k = sqrt(E * N),\n"
                "where E is the embedding dimension and N is the number of valid observations in the prediction set.");
@@ -2023,8 +2047,8 @@ Rcpp::List RcppGPC4Grid(
 
   std::vector<std::vector<double>> xMatrix_cpp(xMatrix.nrow(), std::vector<double>(xMatrix.ncol()));
   std::vector<std::vector<double>> yMatrix_cpp(yMatrix.nrow(), std::vector<double>(yMatrix.ncol()));
-  double validCellNum = 0;
 
+  size_t validCellNum = 0;
   for (int i = 0; i < yMatrix.nrow(); ++i) {
     for (int j = 0; j < yMatrix.ncol(); ++j) {
       xMatrix_cpp[i][j] = xMatrix(i, j);
@@ -2082,7 +2106,7 @@ Rcpp::List RcppGPC4Grid(
 
   // --- Validate parameters --------------------------------------------------
 
-  if (b < 2 || b > validCellNum)
+  if (b < 2 || static_cast<size_t>(b) > validCellNum)
     Rcpp::stop("k cannot be less than or equal to 2 or greater than the number of non-NA values.");
 
   // --- Generate embeddings --------------------------------------------------
@@ -2189,8 +2213,8 @@ Rcpp::DataFrame RcppGPCRobust4Grid(
 
   std::vector<std::vector<double>> xMatrix_cpp(xMatrix.nrow(), std::vector<double>(xMatrix.ncol()));
   std::vector<std::vector<double>> yMatrix_cpp(yMatrix.nrow(), std::vector<double>(yMatrix.ncol()));
-  double validCellNum = 0;
 
+  size_t validCellNum = 0;
   for (int i = 0; i < yMatrix.nrow(); ++i) {
     for (int j = 0; j < yMatrix.ncol(); ++j) {
       xMatrix_cpp[i][j] = xMatrix(i, j);
@@ -2278,7 +2302,7 @@ Rcpp::DataFrame RcppGPCRobust4Grid(
 
   // --- Validate parameters --------------------------------------------------
 
-  if (b < 2 || b > validCellNum)
+  if (b < 2 || static_cast<size_t>(b) > validCellNum)
     Rcpp::stop("k cannot be less than or equal to 2 or greater than the number of non-NA values.");
 
   // --- Generate embeddings --------------------------------------------------
@@ -2402,42 +2426,44 @@ Rcpp::NumericVector RcppSGCSingle4Grid(const Rcpp::NumericMatrix& x,
 
   // Convert lib to a fundamental C++ data type
   int lib_dim = lib.ncol();
-  std::vector<std::pair<int, int>> lib_std(lib.nrow());
+  std::vector<std::pair<int, int>> lib_std;
+  lib_std.reserve(lib.nrow());
 
   if (lib_dim == 1){
     for (int i = 0; i < lib.nrow(); ++i) {
       std::vector<int> rowcolnum = RowColFromGrid(lib(i, 0) - 1, numCols);
       if (!std::isnan(xmat[rowcolnum[0]][rowcolnum[1]]) &&
           !std::isnan(ymat[rowcolnum[0]][rowcolnum[1]])){
-        lib_std[i] = std::make_pair(rowcolnum[0], rowcolnum[1]);
+        lib_std.emplace_back(rowcolnum[0], rowcolnum[1]);
       }
     }
   } else {
     for (int i = 0; i < lib.nrow(); ++i) {
       if (!std::isnan(xmat[lib(i, 0) - 1][lib(i, 1) - 1]) &&
           !std::isnan(ymat[lib(i, 0) - 1][lib(i, 1) - 1])){
-        lib_std[i] = std::make_pair(lib(i, 0) - 1, lib(i, 1) - 1);
+        lib_std.emplace_back(lib(i, 0) - 1, lib(i, 1) - 1);
       }
     }
   }
 
   // Convert pred to a fundamental C++ data type
   int pred_dim = pred.ncol();
-  std::vector<std::pair<int, int>> pred_std(pred.nrow());
+  std::vector<std::pair<int, int>> pred_std;
+  pred_std.reserve(pred.nrow());
 
   if (pred_dim == 1){
     for (int i = 0; i < pred.nrow(); ++i) {
       std::vector<int> rowcolnum = RowColFromGrid(pred(i, 0) - 1, numCols);
       if (!std::isnan(xmat[rowcolnum[0]][rowcolnum[1]]) &&
           !std::isnan(ymat[rowcolnum[0]][rowcolnum[1]])){
-        pred_std[i] = std::make_pair(rowcolnum[0], rowcolnum[1]);
+        pred_std.emplace_back(rowcolnum[0], rowcolnum[1]);
       }
     }
   } else {
     for (int i = 0; i < pred.nrow(); ++i) {
       if (!std::isnan(xmat[pred(i, 0) - 1][pred(i, 1) - 1]) &&
           !std::isnan(ymat[pred(i, 0) - 1][pred(i, 1) - 1])){
-        pred_std[i] = std::make_pair(pred(i, 0) - 1, pred(i, 1) - 1);
+        pred_std.emplace_back(pred(i, 0) - 1, pred(i, 1) - 1);
       }
     }
   }
@@ -2498,42 +2524,44 @@ Rcpp::NumericVector RcppSGC4Grid(const Rcpp::NumericMatrix& x,
 
   // Convert lib to a fundamental C++ data type
   int lib_dim = lib.ncol();
-  std::vector<std::pair<int, int>> lib_std(lib.nrow());
+  std::vector<std::pair<int, int>> lib_std;
+  lib_std.reserve(lib.nrow());
 
   if (lib_dim == 1){
     for (int i = 0; i < lib.nrow(); ++i) {
       std::vector<int> rowcolnum = RowColFromGrid(lib(i, 0) - 1, numCols);
       if (!std::isnan(xmat[rowcolnum[0]][rowcolnum[1]]) &&
           !std::isnan(ymat[rowcolnum[0]][rowcolnum[1]])){
-        lib_std[i] = std::make_pair(rowcolnum[0], rowcolnum[1]);
+        lib_std.emplace_back(rowcolnum[0], rowcolnum[1]);
       }
     }
   } else {
     for (int i = 0; i < lib.nrow(); ++i) {
       if (!std::isnan(xmat[lib(i, 0) - 1][lib(i, 1) - 1]) &&
           !std::isnan(ymat[lib(i, 0) - 1][lib(i, 1) - 1])){
-        lib_std[i] = std::make_pair(lib(i, 0) - 1, lib(i, 1) - 1);
+        lib_std.emplace_back(lib(i, 0) - 1, lib(i, 1) - 1);
       }
     }
   }
 
   // Convert pred to a fundamental C++ data type
   int pred_dim = pred.ncol();
-  std::vector<std::pair<int, int>> pred_std(pred.nrow());
+  std::vector<std::pair<int, int>> pred_std;
+  pred_std.reserve(pred.nrow());
 
   if (pred_dim == 1){
     for (int i = 0; i < pred.nrow(); ++i) {
       std::vector<int> rowcolnum = RowColFromGrid(pred(i, 0) - 1, numCols);
       if (!std::isnan(xmat[rowcolnum[0]][rowcolnum[1]]) &&
           !std::isnan(ymat[rowcolnum[0]][rowcolnum[1]])){
-        pred_std[i] = std::make_pair(rowcolnum[0], rowcolnum[1]);
+        pred_std.emplace_back(rowcolnum[0], rowcolnum[1]);
       }
     }
   } else {
     for (int i = 0; i < pred.nrow(); ++i) {
       if (!std::isnan(xmat[pred(i, 0) - 1][pred(i, 1) - 1]) &&
           !std::isnan(ymat[pred(i, 0) - 1][pred(i, 1) - 1])){
-        pred_std[i] = std::make_pair(pred(i, 0) - 1, pred(i, 1) - 1);
+        pred_std.emplace_back(pred(i, 0) - 1, pred(i, 1) - 1);
       }
     }
   }
@@ -2541,6 +2569,7 @@ Rcpp::NumericVector RcppSGC4Grid(const Rcpp::NumericMatrix& x,
   // Convert block to a fundamental C++ data type
   int b_dim = block.ncol();
   std::vector<int> b_std;
+  b_std.reserve(block.nrow());
   if (b_dim == 1){
     for (int i = 0; i < block.nrow(); ++i) {
       b_std.push_back(block(i, 0));
