@@ -7,7 +7,10 @@
 #include <cmath>
 #include "CppStats.h"
 #include "SimplexProjection.h"
-#include <RcppThread.h>
+// Note: <RcppThread.h> is intentionally excluded from this header to avoid
+//       unnecessary Rcpp dependencies and potential header inclusion order
+//       issues (e.g., R.h being included before Rcpp headers). It should only
+//       be included in the corresponding .cpp implementation file.
 
 /**
  * Computes the multi-view embedding by evaluating multiple feature embeddings using simplex projection,
