@@ -13,7 +13,10 @@
 #include "SimplexProjection.h"
 #include "SMap.h"
 #include "DataStruct.h"
-#include <RcppThread.h>
+// Note: <RcppThread.h> is intentionally excluded from this header to avoid
+//       unnecessary Rcpp dependencies and potential header inclusion order
+//       issues (e.g., R.h being included before Rcpp headers). It should only
+//       be included in the corresponding .cpp implementation file.
 
 /**
  * @brief Computes the partial correlation between a spatial cross-sectional series and its prediction
