@@ -6,7 +6,10 @@
 #include "CppLatticeUtils.h"
 #include "Entropy.h"
 #include "SpatialBlockBootstrap.h"
-#include <RcppThread.h>
+// Note: <RcppThread.h> is intentionally excluded from this header to avoid
+//       unnecessary Rcpp dependencies and potential header inclusion order
+//       issues (e.g., R.h being included before Rcpp headers). It should only
+//       be included in the corresponding .cpp implementation file.
 
 /**
  * @brief Computes directional spatial granger causality between two spatial variables
@@ -127,7 +130,7 @@ std::vector<double> SGC4Lattice(
     int threads,
     int boot = 399,
     double base = 2,
-    unsigned int seed = 42,
+    unsigned long long seed = 42,
     bool symbolize = true,
     bool normalize = false,
     bool progressbar = true
