@@ -5,21 +5,29 @@
 #include <limits>
 #include <vector>
 #include <numeric>
+#include <random>    // std::mt19937_64, std::normal_distribution
+#include <algorithm>
+#include "NumericUtils.h"
 #include "CppLatticeUtils.h"
 
 /**
- * @brief Simulate a univariate Spatial Logistic Map (SLM) over lattice-structured data.
+ * @brief Simulate a univariate Spatial Logistic Map (SLM) over lattice-structured data,
+ *        with optional additive Gaussian white noise.
  *
- * This function performs time-stepped simulations of the Spatial Logistic Map
- * on a lattice data where each spatial unit evolves based on its own value and
- * the average of its k nearest neighbors.
+ * This function performs time-stepped simulations of the Spatial Logistic Map (SLM)
+ * on lattice data, where each spatial unit evolves based on its own value and
+ * the average of its k nearest neighbors. Optionally, zero-mean Gaussian noise is added
+ * to each state update to represent observational or dynamical uncertainty.
  *
- * @param vec                Initial values of the lattice data (e.g., population densities).
- * @param nb                 Neighbor list for each lattice unit (e.g., rook or queen adjacency).
+ * @param vec                Initial values of the lattice data.
+ * @param nb                 Neighbor list for each lattice unit.
  * @param k                  Number of neighbors to consider.
  * @param step               Number of simulation time steps to run.
- * @param alpha              Growth/interaction parameter in the logistic update rule.
+ * @param alpha              Logistic interaction parameter.
+ * @param noise_level        Standard deviation of additive Gaussian noise (default = 0).
+ *                           If set to 0, no noise is applied.
  * @param escape_threshold   Threshold to treat divergent values as invalid (default: 1e10).
+ * @param random_seed        Seed for random number generator (default: 42).
  *
  * @return A 2D vector of simulation results:
  *         Each row corresponds to a spatial unit,
@@ -31,7 +39,9 @@ std::vector<std::vector<double>> SLMUni4Lattice(
     size_t k,
     size_t step,
     double alpha,
-    double escape_threshold = 1e10
+    double noise_level = 0.0,
+    double escape_threshold = 1e10,
+    unsigned long long random_seed = 42
 );
 
 /**
@@ -53,6 +63,8 @@ std::vector<std::vector<double>> SLMUni4Lattice(
  *   - Variable 2 evolves based on its neighbors of variable 2 and inhibition from variable 1
  *     (local or neighbor-averaged).
  *
+ * Zero-mean Gaussian noise may be added to each update to represent uncertainty.
+ *
  * @param vec1               Initial values of the first spatial variable (e.g., species A density).
  * @param vec2               Initial values of the second spatial variable (e.g., species B density).
  * @param nb                 Neighbor list for each spatial unit (e.g., rook or queen adjacency).
@@ -63,7 +75,9 @@ std::vector<std::vector<double>> SLMUni4Lattice(
  * @param beta12             Cross-inhibition coefficient from variable 1 to variable 2.
  * @param beta21             Cross-inhibition coefficient from variable 2 to variable 1.
  * @param interact           Interaction type (0 = local interaction, 1 = neighbor-averaged interaction).
+ * @param noise_level        Std. dev. of Gaussian noise (default = 0; no noise applied if = 0).
  * @param escape_threshold   Threshold to treat divergent values as invalid (default: 1e10).
+ * @param random_seed        RNG seed (default: 42).
  *
  * @return A 3D vector of simulation results:
  *         - First dimension: variable index (0 for vec1, 1 for vec2),
@@ -81,7 +95,9 @@ std::vector<std::vector<std::vector<double>>> SLMBi4Lattice(
     double beta12,
     double beta21,
     int interact = 0,
-    double escape_threshold = 1e10
+    double noise_level = 0.0,
+    double escape_threshold = 1e10,
+    unsigned long long random_seed = 42
 );
 
 /**
@@ -116,7 +132,10 @@ std::vector<std::vector<std::vector<double>>> SLMBi4Lattice(
  * @param interact           Type of cross-variable interaction:
  *                           0 = use local values (default behavior for k>0),
  *                           1 = use neighbor averages instead.
- * @param escape_threshold   Threshold beyond which values are treated as divergent (default: 1e10).
+ * @param noise_level        Standard deviation of additive Gaussian noise (default = 0).
+ *                           If set to 0, no noise is applied.
+ * @param escape_threshold   Threshold to treat divergent values as invalid (default: 1e10).
+ * @param random_seed        Seed for random number generator (default: 42).
  *
  * @return A 3D vector of simulation results:
  *         - First dimension: variable index (0 for vec1, 1 for vec2, 2 for vec3),
@@ -140,7 +159,9 @@ std::vector<std::vector<std::vector<double>>> SLMTri4Lattice(
     double beta31,
     double beta32,
     int interact = 0,
-    double escape_threshold = 1e10
+    double noise_level = 0.0,
+    double escape_threshold = 1e10,
+    unsigned long long random_seed = 42
 );
 
 #endif // SLM4Lattice_H
